"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const loggerhythm_1 = require("loggerhythm");
const uuid = require("uuid");
const logger = loggerhythm_1.Logger.createLogger('pprocessengine:external_task:worker');
class ExternalTaskWorker {
    constructor(externalTaskApi) {
        // eslint-disable-next-line @typescript-eslint/member-naming
        this._workerId = uuid.v4();
        this.lockDuration = 30000;
        this.externalTaskApi = undefined;
        this.externalTaskApi = externalTaskApi;
    }
    get workerId() {
        return this._workerId;
    }
    async waitForAndHandle(identity, topic, maxTasks, longpollingTimeout, handleAction) {
        const keepPolling = true;
        while (keepPolling) {
            const externalTasks = await this.fetchAndLockExternalTasks(identity, topic, maxTasks, longpollingTimeout);
            if (externalTasks.length === 0) {
                await this.sleep(1000);
                continue;
            }
            const executeTaskPromises = [];
            for (const externalTask of externalTasks) {
                executeTaskPromises.push(this.executeExternalTask(identity, externalTask, handleAction));
            }
            await Promise.all(executeTaskPromises);
        }
    }
    async fetchAndLockExternalTasks(identity, topic, maxTasks, longpollingTimeout) {
        try {
            return await this
                .externalTaskApi
                .fetchAndLockExternalTasks(identity, this.workerId, topic, maxTasks, longpollingTimeout, this.lockDuration);
        }
        catch (error) {
            logger.error('An error occured during fetchAndLock!', 'This can happen, if the tasks were already locked by another worker, before this worker could apply its own lock.', error);
            // Returning an empty Array here, since "waitForAndHandle" already implements a timeout, in case no tasks are available for processing.
            // No need to do that twice.
            return [];
        }
    }
    async executeExternalTask(identity, externalTask, handleAction) {
        try {
            const lockExtensionBuffer = 5000;
            const interval = setInterval(async () => this.extendLocks(identity, externalTask), this.lockDuration - lockExtensionBuffer);
            const result = await handleAction(externalTask);
            clearInterval(interval);
            await result.sendToExternalTaskApi(this.externalTaskApi, identity, this.workerId);
        }
        catch (error) {
            logger.error('Failed to execute ExternalTask!', error);
            await this.externalTaskApi.handleServiceError(identity, this.workerId, externalTask.id, error.message, '');
        }
    }
    async extendLocks(identity, externalTask) {
        try {
            await this.externalTaskApi.extendLock(identity, this.workerId, externalTask.id, this.lockDuration);
        }
        catch (error) {
            // This can happen, if the lock-extension was performed after the task was already finished.
            // Since this isn't really an error, a warning suffices here.
            logger.warn(`An error occured while trying to extend the lock for ExternalTask ${externalTask.id}`, error);
        }
    }
    async sleep(milliseconds) {
        return new Promise((resolve) => {
            setTimeout(() => { resolve(); }, milliseconds);
        });
    }
}
exports.ExternalTaskWorker = ExternalTaskWorker;
//# sourceMappingURL=external_task_worker.js.map